package com.example.demo.component

import androidx.compose.foundation.border
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.ColumnScope
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.navigationBarsPadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.statusBarsPadding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.BasicTextField
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.ModalBottomSheet
import androidx.compose.material3.ModalBottomSheetDefaults
import androidx.compose.material3.ModalBottomSheetProperties
import androidx.compose.material3.SheetState
import androidx.compose.material3.Text
import androidx.compose.material3.rememberModalBottomSheetState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.onFocusChanged
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.Shape
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.VisualTransformation
import androidx.compose.ui.unit.TextUnit
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.example.demo.ui.theme.PurpleGrey40


@Composable
fun TextComponent(
    modifier: Modifier = Modifier,
    text: String?,
    color: Color = Color.Unspecified,
    fontFamily: FontFamily? = null,
    fontSize: TextUnit = TextUnit.Unspecified,
    style: TextStyle = TextStyle(),
) {
    Text(
        text = text ?: "",
        fontSize = fontSize,
        style = style,
        color = color,
        modifier = modifier,
        fontFamily = fontFamily
    )
}


@Composable
fun TextFieldComponent(
    modifier: Modifier = Modifier,
    value: String,
    placeholder: String = "",
    singleLine: Boolean = true,
    isError: Boolean = false,
    keyboardActions: KeyboardActions = KeyboardActions(),
    shape: Shape = MaterialTheme.shapes.extraSmall,
    suffix: @Composable (() -> Unit)? = null,
    prefix: @Composable (() -> Unit)? = null,
    visualTransformation: VisualTransformation = VisualTransformation.None,
    textStyle: TextStyle = MaterialTheme.typography.bodySmall.copy(color = MaterialTheme.colorScheme.secondary),
    keyboardOptions: KeyboardOptions = KeyboardOptions(
        keyboardType = KeyboardType.Text,
        imeAction = ImeAction.Next
    ),
    onValueChange: (String) -> Unit
) {
    var isFocused by remember { mutableStateOf(false) }
    val borderColor = when {
        isError -> Color.Red
        isFocused -> MaterialTheme.colorScheme.primary
        else -> PurpleGrey40
    }

    BasicTextField(
        modifier = modifier
            .fillMaxWidth()
            .padding(top = 6.dp)
            .border(
                width = 1.dp,
                color = borderColor,
                shape = shape
            )
            .onFocusChanged {
                isFocused = it.isFocused
            }
            .padding(horizontal = 12.dp, vertical = 14.dp),
        value = value,
        visualTransformation = visualTransformation,
        textStyle = textStyle,
        onValueChange = onValueChange,
        singleLine = singleLine,
        keyboardActions = keyboardActions,
        keyboardOptions = keyboardOptions,
        decorationBox = { innerTextField ->
            Row(verticalAlignment = Alignment.CenterVertically) {
                prefix?.invoke()
                Box(
                    modifier = Modifier
                        .weight(1F)
                        .padding(horizontal = 10.dp)
                ) {
                    if (value.isEmpty()) {
                        TextComponent(
                            text = placeholder,
                            fontSize = 14.sp,
                        )
                    }
                    innerTextField()
                }
                suffix?.invoke()
            }
        }
    )
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun BottomSheetComponent(
    modifier: Modifier = Modifier,
    onDismiss: () -> Unit,
    shape: Shape = RoundedCornerShape(topEnd = 24.dp, topStart = 24.dp),
    properties: ModalBottomSheetProperties = ModalBottomSheetDefaults.properties(),
    sheetState: SheetState = rememberModalBottomSheetState(skipPartiallyExpanded = true),
    content: @Composable (ColumnScope.() -> Unit)
) {

    ModalBottomSheet(
        onDismissRequest = onDismiss, sheetState = sheetState,
        modifier = modifier
            .statusBarsPadding()
            .navigationBarsPadding(),
        dragHandle = null,
        properties = properties,
        shape = shape,
        containerColor = MaterialTheme.colorScheme.background,
        content = content
    )
}